==> tmgmt_deepl-2.0.0.tar.gz: tmgmt_deepl/src/Plugin/tmgmt/Translator/DeeplProTranslator.php <==
<?php

namespace Drupal\tmgmt_deepl\Plugin\tmgmt\Translator;

use Drupal\Component\Utility\Html;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\tmgmt\ContinuousTranslatorInterface;
use Drupal\tmgmt\Data;
use Drupal\tmgmt\Entity\Job;
use Drupal\tmgmt\Entity\Translator;
use Drupal\tmgmt\JobInterface;
use Drupal\tmgmt\TMGMTException;
use Drupal\tmgmt\Translator\AvailableResult;
use Drupal\tmgmt\TranslatorInterface;
use Drupal\tmgmt\TranslatorPluginBase;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\BadResponseException;
use GuzzleHttp\Psr7\Request;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * DeepL Pro translator plugin.
 *
 * @TranslatorPlugin(
 *   id = "deepl_pro",
 *   label = @Translation("DeepL Pro"),
 *   description = @Translation("DeepL Pro Translator service."),
 *   ui = "Drupal\tmgmt_deepl\DeeplProTranslatorUi",
 *   logo = "icons/deepl.svg",
 * )
 */
class DeeplProTranslator extends TranslatorPluginBase implements ContainerFactoryPluginInterface, ContinuousTranslatorInterface {

  use StringTranslationTrait;
  /**
   * Translation service URL.
   *
   * @var string
   */
  protected $translatorUrl = 'https://api.deepl.com/v2/translate';

  /**
   * Translation usage service URL.
   *
   * @var string
   */
  protected $translatorUsageUrl = 'https://api.deepl.com/v2/usage';

  /**
   * Name of parameter that contains source string to be translated.
   *
   * @var string
   */
  protected $qParamName = 'text';

  /**
   * Max number of text queries for translation sent in one request.
   *
   * @var int
   */
  protected $qChunkSize = 50;

  /**
   * Guzzle HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $client;

  /**
   * TMGMT data service.
   *
   * @var \Drupal\tmgmt\Data
   */
  protected $tmgmtData;

  /**
   * Constructs a DeeplProTranslator object.
   *
   * @param \GuzzleHttp\ClientInterface $client
   *   The Guzzle HTTP client.
   * @param \Drupal\tmgmt\Data $tmgmt_data
   *   The Guzzle HTTP client.
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   */
  public function __construct(ClientInterface $client, Data $tmgmt_data, array $configuration, $plugin_id, array $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->client = $client;
    $this->tmgmtData = $tmgmt_data;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $container->get('http_client'),
      $container->get('tmgmt.data'),
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::checkAvailable().
   */
  public function checkAvailable(TranslatorInterface $translator) {
    if ($translator->getSetting('auth_key')) {
      return AvailableResult::yes();
    }

    return AvailableResult::no($this->t('@translator is not available. Make sure it is properly <a href=:configured>configured</a>.', [
      '@translator' => $translator->label(),
      ':configured' => $translator->toUrl(),
    ]));
  }

  /**
   * Implements TMGMTTranslatorPluginControllerInterface::requestTranslation().
   */
  public function requestTranslation(JobInterface $job) {
    $this->requestJobItemsTranslation($job->getItems());
    if (!$job->isRejected()) {
      $job->submitted('The translation job has been submitted.');
    }
  }

  /**
   * Helper method to do translation request.
   *
   * @param \Drupal\tmgmt\Entity\Job $job
   *   TMGMT Job to be used for translation.
   * @param array|string $text
   *   Text/texts to be translated.
   *
   * @return array
   *   Userialized JSON containing translated texts.
   */
  protected function deeplProRequestTranslation(Job $job, $text) {
    $translator = $job->getTranslator();

    // Fix source language mapping.
    $source_lang = $this->fixSourceLanguageMappings($job->getRemoteSourceLanguage());

    // Build query params.
    $query_params = [
      'source_lang' => $source_lang,
      'target_lang' => $job->getRemoteTargetLanguage(),
      $this->qParamName => $text,
    ];
    return $this->doRequest($translator, $query_params);
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultRemoteLanguagesMappings() {
    return [
      'bg' => 'BG',
      'cs' => 'CS',
      'da' => 'DA',
      'de' => 'DE',
      'el' => 'EL',
      'en' => 'EN',
      'es' => 'ES',
      'et' => 'ET',
      'fi' => 'FI',
      'fr' => 'FR',
      'hu' => 'HU',
      'it' => 'IT',
      'ja' => 'JA',
      'lt' => 'LT',
      'lv' => 'LV',
      'nl' => 'NL',
      'pl' => 'PL',
      'pt-br' => 'PT-BR',
      'pt-pt' => 'PT-PT',
      'ro' => 'RO',
      'ru' => 'RU',
      'sk' => 'SK',
      'sl' => 'SL',
      'sv' => 'SV',
      'zh' => 'ZH',
    ];
  }

  /**
   * Overrides getSupportedRemoteLanguages().
   */
  public function getSupportedRemoteLanguages(TranslatorInterface $translator) {
    // Pre-defined array of available languages.
    return [
      'BG' => $this->t('Bulgarian'),
      'CS' => $this->t('Czech'),
      'DA' => $this->t('Danish'),
      'DE' => $this->t('German'),
      'EL' => $this->t('Greek'),
      'EN-GB' => $this->t('English (British)'),
      'EN-US' => $this->t('English (American)'),
      'EN' => $this->t('English'),
      'ES' => $this->t('Spanish'),
      'ET' => $this->t('Estonian'),
      'FI' => $this->t('Finnish'),
      'FR' => $this->t('French'),
      'HU' => $this->t('Hungarian'),
      'IT' => $this->t('Italian'),
      'JA' => $this->t('Japanese'),
      'LT' => $this->t('Lithuanian'),
      'LV' => $this->t('Latvian'),
      'NL' => $this->t('Dutch'),
      'PL' => $this->t('Polish'),
      'PT-PT' => $this->t('Portuguese (excluding Brazilian Portuguese)'),
      'PT-BR' => $this->t('Portuguese (Brazilian)'),
      'PT' => $this->t('Portuguese (deprecated, select PT-PT or PT-BR instead)'),
      'RO' => $this->t('Romanian'),
      'RU' => $this->t('Russian'),
      'SK' => $this->t('Slovak'),
      'SL' => $this->t('Slovenian'),
      'SV' => $this->t('Swedish'),
      'ZH' => $this->t('Chinese'),
    ];
  }

  /**
   * Overrides getSupportedTargetLanguages().
   */
  public function getSupportedTargetLanguages(TranslatorInterface $translator, $source_language) {

    $languages = $this->getSupportedRemoteLanguages($translator);

    // There are no language pairs, any supported language can be translated
    // into the others. If the source language is part of the languages,
    // then return them all, just remove the source language.
    if (array_key_exists($source_language, $languages)) {
      unset($languages[$source_language]);
      return $languages;
    }

    return [];
  }

  /**
   * Update source language mapping, cause DeepL doesn't support all sources as
   * target languages.
   *
   * @param string $source_lang
   *  The selected source language of the job item.
   *
   * @return string
   *  Fixed language mapping based on DeepL specification.
   */
  public function fixSourceLanguageMappings($source_lang) {
    $language_mapping = [
      'EN-GB' => 'EN',
      'EN-US' => 'EN'
    ];

    if (isset($language_mapping[$source_lang])) {
      return $language_mapping[$source_lang];
    }

    return $source_lang;
  }


  /**
   * Overrides TMGMTDefaultTranslatorPluginController::hasCheckoutSettings().
   */
  public function hasCheckoutSettings(JobInterface $job) {
    return FALSE;
  }

  /**
   * Local method to do request to DeepL Pro Translate service.
   *
   * @param \Drupal\tmgmt\Entity\Translator $translator
   *   The translator entity to get the settings from.
   * @param array $query_params
   *   (Optional) Additional query params to be passed into the request.
   * @param array $options
   *   (Optional) Additional options that will passed to drupal_http_request().
   *
   * @return array|object
   *   Unserialized JSON response from DeepL Pro.
   *
   * @throws TMGMTException
   *   - Unable to connect to the DeepL Pro Service
   *   - Error returned by the DeepL Pro Service.
   */
  protected function doRequest(Translator $translator, array $query_params = [], array $options = []) {
    // Get custom URL for testing purposes, if available.
    $custom_url = $translator->getSetting('url');
    $url = $custom_url ? $custom_url : $this->translatorUrl;

    // Define headers.
    $headers = [
      'Content-Type' => 'application/x-www-form-urlencoded',
    ];

    // Build the query.
    $query_string = '';
    $query_string .= '&auth_key=' . $translator->getSetting('auth_key');

    // Add text to be translated.
    if (isset($query_params[$this->qParamName])) {
      foreach ($query_params[$this->qParamName] as $source_text) {
        // We use rawurlencode here, cause urlencode is not encoding blanks correctly.
        $query_string .= '&text=' . rawurlencode($source_text);
      }
    }
    // Add source language.
    if (isset($query_params['source_lang'])) {
      $query_string .= '&source_lang=' . $query_params['source_lang'];
    }

    // Add target language.
    if (isset($query_params['target_lang'])) {
      $query_string .= '&target_lang=' . $query_params['target_lang'];
    }

    // Split sentences.
    $query_string .= '&split_sentences=' . $translator->getSetting('split_sentences');

    // Formality.
    $query_string .= '&formality=' . $translator->getSetting('formality');

    // Preserve formatting.
    $query_string .= '&preserve_formatting=' . $translator->getSetting('preserve_formatting');

    // Tag handling.
    $tag_handling = $translator->getSetting('tag_handling');
    $query_string .= '&tag_handling=' . $tag_handling;

    if ($tag_handling == 'xml') {
      // Non splitting tags.
      if (!empty($translator->getSetting('non_splitting_tags'))) {
        $query_string .= '&non_splitting_tags=' . urlencode($translator->getSetting('non_splitting_tags'));
      }

      // Splitting tags.
      if (!empty($translator->getSetting('splitting_tags'))) {
        $query_string .= '&splitting_tags=' . urlencode($translator->getSetting('splitting_tags'));
      }

      // Ignore tags.
      if (!empty($translator->getSetting('ignore_tags'))) {
        $query_string .= '&ignore_tags=' . urlencode($translator->getSetting('ignore_tags'));
      }

      // Automatic outline detection.
      $query_string .= '&outline_detection=' . $translator->getSetting('outline_detection');
    }

    // Build request object.
    $request = new Request('POST', $url, $headers, $query_string);

    // Send the request with the query.
    try {
      $response = $this->client->send($request);
    }
    catch (BadResponseException $e) {
      $error = json_decode($e->getResponse()->getBody(), TRUE);
      throw new TMGMTException('DeepL Pro service returned following error: @error', ['@error' => $error['error']['message']]);
    }

    // Process the JSON result into array.
    return json_decode($response->getBody(), TRUE);
  }

  /**
   * Provide translatorUrl for override  in automated testing.
   *
   * @param string $translator_url
   *   Url of translator service.
   */
  final public function setTranslatorUrl($translator_url) {
    $this->translatorUrl = $translator_url;
  }

  /**
   * {@inheritdoc}
   */
  public function requestJobItemsTranslation(array $job_items) {
    /** @var \Drupal\tmgmt\Entity\Job $job */
    $job = reset($job_items)->getJob();
    foreach ($job_items as $job_item) {
      if ($job->isContinuous()) {
        $job_item->active();
      }
      // Pull the source data array through the job and flatten it.
      $data = $this->tmgmtData->filterTranslatable($job_item->getData());

      $translation = [];
      $q = [];
      $keys_sequence = [];
      $i = 0;

      // Build DeepL Pro q param and preserve initial array keys.
      foreach ($data as $key => $value) {
        $q[] = $value['#text'];
        $keys_sequence[] = $key;
      }

      try {

        // Split $q into chunks of self::qChunkSize.
        foreach (array_chunk($q, $this->qChunkSize) as $_q) {

          // Get translation from DeepL Pro.
          $result = $this->deeplProRequestTranslation($job, $_q);

          // Collect translated texts with use of initial keys.
          foreach ($result['translations'] as $translated) {
            $translation[$keys_sequence[$i]]['#text'] = $translated['text'];
            $i++;
          }
        }

        // Save the translated data through the job.
        // NOTE that this line of code is reached only in case all translation
        // requests succeeded.
        $job_item->addTranslatedData($this->tmgmtData->unflatten($translation));

      }
      catch (TMGMTException $e) {
        $job->rejected('Translation has been rejected with following error: @error',
          ['@error' => $e->getMessage()], 'error');
      }
    }
  }

  /**
   * Local method to do request to DeepL Pro Usage service.
   *
   * @param \Drupal\tmgmt\Entity\Translator $translator
   *   The translator entity to get the settings from.
   *
   * @return array|object|integer
   *   Unserialized JSON response from DeepL Pro or error code.
   *
   * @throws TMGMTException
   *   - Unable to connect to the DeepL Pro Service
   *   - Error returned by the DeepL Pro Service.
   */
  public function getUsageData(Translator $translator) {
    // Set custom data for testing purposes, if available.
    $custom_usage_url = $translator->getSetting('url_usage');
    $custom_auth_key = $translator->getSetting('auth_key');

    $url = $custom_usage_url ? $custom_usage_url : $this->translatorUsageUrl;
    $auth_key = $custom_auth_key ? $custom_auth_key : $translator->getSetting('auth_key');

    // Prepare Guzzle Object.
    $request = new Request('GET', $url);

    // Build the query.
    $query_string = '&auth_key=' . $auth_key;

    // Send the request with the query.
    try {
      $response = $this->client->send($request, ['query' => $query_string]);
    }
    catch (BadResponseException $e) {
      return json_decode($e->getCode());
    }

    // Process the JSON result into array.
    return json_decode($response->getBody(), TRUE);
  }

}

==> tmgmt_deepl-2.0.1.tar.gz: tmgmt_deepl/src/Plugin/tmgmt/Translator/DeeplProTranslator.php <==
<?php

namespace Drupal\tmgmt_deepl\Plugin\tmgmt\Translator;

use Drupal\Component\Utility\Html;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\tmgmt\ContinuousTranslatorInterface;
use Drupal\tmgmt\Data;
use Drupal\tmgmt\Entity\Job;
use Drupal\tmgmt\Entity\Translator;
use Drupal\tmgmt\JobInterface;
use Drupal\tmgmt\TMGMTException;
use Drupal\tmgmt\Translator\AvailableResult;
use Drupal\tmgmt\TranslatorInterface;
use Drupal\tmgmt\TranslatorPluginBase;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\BadResponseException;
use GuzzleHttp\Psr7\Request;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * DeepL Pro translator plugin.
 *
 * @TranslatorPlugin(
 *   id = "deepl_pro",
 *   label = @Translation("DeepL Pro"),
 *   description = @Translation("DeepL Pro Translator service."),
 *   ui = "Drupal\tmgmt_deepl\DeeplProTranslatorUi",
 *   logo = "icons/deepl.svg",
 * )
 */
class DeeplProTranslator extends TranslatorPluginBase implements ContainerFactoryPluginInterface, ContinuousTranslatorInterface {

  use StringTranslationTrait;
  /**
   * Translation service URL.
   *
   * @var string
   */
  protected $translatorUrl = 'https://api.deepl.com/v2/translate';

  /**
   * Translation usage service URL.
   *
   * @var string
   */
  protected $translatorUsageUrl = 'https://api.deepl.com/v2/usage';

  /**
   * Name of parameter that contains source string to be translated.
   *
   * @var string
   */
  protected $qParamName = 'text';

  /**
   * Max number of text queries for translation sent in one request.
   *
   * @var int
   */
  protected $qChunkSize = 50;

  /**
   * Guzzle HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $client;

  /**
   * TMGMT data service.
   *
   * @var \Drupal\tmgmt\Data
   */
  protected $tmgmtData;

  /**
   * Constructs a DeeplProTranslator object.
   *
   * @param \GuzzleHttp\ClientInterface $client
   *   The Guzzle HTTP client.
   * @param \Drupal\tmgmt\Data $tmgmt_data
   *   The Guzzle HTTP client.
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   */
  public function __construct(ClientInterface $client, Data $tmgmt_data, array $configuration, $plugin_id, array $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->client = $client;
    $this->tmgmtData = $tmgmt_data;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $container->get('http_client'),
      $container->get('tmgmt.data'),
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::checkAvailable().
   */
  public function checkAvailable(TranslatorInterface $translator) {
    if ($translator->getSetting('auth_key')) {
      return AvailableResult::yes();
    }

    return AvailableResult::no($this->t('@translator is not available. Make sure it is properly <a href=:configured>configured</a>.', [
      '@translator' => $translator->label(),
      ':configured' => $translator->toUrl(),
    ]));
  }

  /**
   * Implements TMGMTTranslatorPluginControllerInterface::requestTranslation().
   */
  public function requestTranslation(JobInterface $job) {
    $this->requestJobItemsTranslation($job->getItems());
    if (!$job->isRejected()) {
      $job->submitted('The translation job has been submitted.');
    }
  }

  /**
   * Helper method to do translation request.
   *
   * @param \Drupal\tmgmt\Entity\Job $job
   *   TMGMT Job to be used for translation.
   * @param array|string $text
   *   Text/texts to be translated.
   *
   * @return array
   *   Userialized JSON containing translated texts.
   */
  protected function deeplProRequestTranslation(Job $job, $text) {
    $translator = $job->getTranslator();

    // Fix source language mapping.
    $source_lang = $this->fixSourceLanguageMappings($job->getRemoteSourceLanguage());

    // Build query params.
    $query_params = [
      'source_lang' => $source_lang,
      'target_lang' => $job->getRemoteTargetLanguage(),
      $this->qParamName => $text,
    ];
    return $this->doRequest($translator, $query_params);
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultRemoteLanguagesMappings() {
    return [
      'bg' => 'BG',
      'cs' => 'CS',
      'da' => 'DA',
      'de' => 'DE',
      'el' => 'EL',
      'en' => 'EN',
      'es' => 'ES',
      'et' => 'ET',
      'fi' => 'FI',
      'fr' => 'FR',
      'hu' => 'HU',
      'it' => 'IT',
      'ja' => 'JA',
      'lt' => 'LT',
      'lv' => 'LV',
      'nl' => 'NL',
      'pl' => 'PL',
      'pt-br' => 'PT-BR',
      'pt-pt' => 'PT-PT',
      'ro' => 'RO',
      'ru' => 'RU',
      'sk' => 'SK',
      'sl' => 'SL',
      'sv' => 'SV',
      'zh' => 'ZH',
    ];
  }

  /**
   * Overrides getSupportedRemoteLanguages().
   */
  public function getSupportedRemoteLanguages(TranslatorInterface $translator) {
    // Pre-defined array of available languages.
    return [
      'BG' => $this->t('Bulgarian'),
      'CS' => $this->t('Czech'),
      'DA' => $this->t('Danish'),
      'DE' => $this->t('German'),
      'EL' => $this->t('Greek'),
      'EN-GB' => $this->t('English (British)'),
      'EN-US' => $this->t('English (American)'),
      'EN' => $this->t('English'),
      'ES' => $this->t('Spanish'),
      'ET' => $this->t('Estonian'),
      'FI' => $this->t('Finnish'),
      'FR' => $this->t('French'),
      'HU' => $this->t('Hungarian'),
      'IT' => $this->t('Italian'),
      'JA' => $this->t('Japanese'),
      'LT' => $this->t('Lithuanian'),
      'LV' => $this->t('Latvian'),
      'NL' => $this->t('Dutch'),
      'PL' => $this->t('Polish'),
      'PT-PT' => $this->t('Portuguese (excluding Brazilian Portuguese)'),
      'PT-BR' => $this->t('Portuguese (Brazilian)'),
      'PT' => $this->t('Portuguese (deprecated, select PT-PT or PT-BR instead)'),
      'RO' => $this->t('Romanian'),
      'RU' => $this->t('Russian'),
      'SK' => $this->t('Slovak'),
      'SL' => $this->t('Slovenian'),
      'SV' => $this->t('Swedish'),
      'ZH' => $this->t('Chinese'),
    ];
  }

  /**
   * Overrides getSupportedTargetLanguages().
   */
  public function getSupportedTargetLanguages(TranslatorInterface $translator, $source_language) {

    $languages = $this->getSupportedRemoteLanguages($translator);

    // There are no language pairs, any supported language can be translated
    // into the others. If the source language is part of the languages,
    // then return them all, just remove the source language.
    if (array_key_exists($source_language, $languages)) {
      unset($languages[$source_language]);
      return $languages;
    }

    return [];
  }

  /**
   * Update source language mapping, cause DeepL doesn't support all sources as
   * target languages.
   *
   * @param string $source_lang
   *  The selected source language of the job item.
   *
   * @return string
   *  Fixed language mapping based on DeepL specification.
   */
  public function fixSourceLanguageMappings($source_lang) {
    $language_mapping = [
      'EN-GB' => 'EN',
      'EN-US' => 'EN'
    ];

    if (isset($language_mapping[$source_lang])) {
      return $language_mapping[$source_lang];
    }

    return $source_lang;
  }


  /**
   * Overrides TMGMTDefaultTranslatorPluginController::hasCheckoutSettings().
   */
  public function hasCheckoutSettings(JobInterface $job) {
    return FALSE;
  }

  /**
   * Local method to do request to DeepL Pro Translate service.
   *
   * @param \Drupal\tmgmt\Entity\Translator $translator
   *   The translator entity to get the settings from.
   * @param array $query_params
   *   (Optional) Additional query params to be passed into the request.
   * @param array $options
   *   (Optional) Additional options that will passed to drupal_http_request().
   *
   * @return array|object
   *   Unserialized JSON response from DeepL Pro.
   *
   * @throws TMGMTException
   *   - Unable to connect to the DeepL Pro Service
   *   - Error returned by the DeepL Pro Service.
   */
  protected function doRequest(Translator $translator, array $query_params = [], array $options = []) {
    // Get custom URL for testing purposes, if available.
    $custom_url = $translator->getSetting('url');
    $url = $custom_url ? $custom_url : $this->translatorUrl;

    // Define headers.
    $headers = [
      'Content-Type' => 'application/x-www-form-urlencoded',
    ];

    // Build the query.
    $query_string = '';
    $query_string .= '&auth_key=' . $translator->getSetting('auth_key');

    // Add text to be translated.
    if (isset($query_params[$this->qParamName])) {
      foreach ($query_params[$this->qParamName] as $source_text) {
        // We use rawurlencode here, cause urlencode is not encoding blanks correctly.
        $query_string .= '&text=' . rawurlencode($source_text);
      }
    }
    // Add source language.
    if (isset($query_params['source_lang'])) {
      $query_string .= '&source_lang=' . $query_params['source_lang'];
    }

    // Add target language.
    if (isset($query_params['target_lang'])) {
      $query_string .= '&target_lang=' . $query_params['target_lang'];
    }

    // Split sentences.
    $query_string .= '&split_sentences=' . $translator->getSetting('split_sentences');

    // Formality.
    $query_string .= '&formality=' . $translator->getSetting('formality');

    // Preserve formatting.
    $query_string .= '&preserve_formatting=' . $translator->getSetting('preserve_formatting');

    // Tag handling.
    $tag_handling = $translator->getSetting('tag_handling');
    $query_string .= '&tag_handling=' . $tag_handling;

    if ($tag_handling == 'xml') {
      // Non splitting tags.
      if (!empty($translator->getSetting('non_splitting_tags'))) {
        $query_string .= '&non_splitting_tags=' . urlencode($translator->getSetting('non_splitting_tags'));
      }

      // Splitting tags.
      if (!empty($translator->getSetting('splitting_tags'))) {
        $query_string .= '&splitting_tags=' . urlencode($translator->getSetting('splitting_tags'));
      }

      // Ignore tags.
      if (!empty($translator->getSetting('ignore_tags'))) {
        $query_string .= '&ignore_tags=' . urlencode($translator->getSetting('ignore_tags'));
      }

      // Automatic outline detection.
      $query_string .= '&outline_detection=' . $translator->getSetting('outline_detection');
    }

    // Build request object.
    $request = new Request('POST', $url, $headers, $query_string);

    // Send the request with the query.
    try {
      $response = $this->client->send($request);
    }
    catch (BadResponseException $e) {
      $error = json_decode($e->getResponse()->getBody(), TRUE);
      throw new TMGMTException('DeepL Pro service returned following error: @error', ['@error' => $error['message']]);
    }

    // Process the JSON result into array.
    return json_decode($response->getBody(), TRUE);
  }

  /**
   * Provide translatorUrl for override  in automated testing.
   *
   * @param string $translator_url
   *   Url of translator service.
   */
  final public function setTranslatorUrl($translator_url) {
    $this->translatorUrl = $translator_url;
  }

  /**
   * {@inheritdoc}
   */
  public function requestJobItemsTranslation(array $job_items) {
    /** @var \Drupal\tmgmt\Entity\Job $job */
    $job = reset($job_items)->getJob();
    foreach ($job_items as $job_item) {
      if ($job->isContinuous()) {
        $job_item->active();
      }
      // Pull the source data array through the job and flatten it.
      $data = $this->tmgmtData->filterTranslatable($job_item->getData());

      $translation = [];
      $q = [];
      $keys_sequence = [];
      $i = 0;

      // Build DeepL Pro q param and preserve initial array keys.
      foreach ($data as $key => $value) {
        $q[] = $value['#text'];
        $keys_sequence[] = $key;
      }

      try {

        // Split $q into chunks of self::qChunkSize.
        foreach (array_chunk($q, $this->qChunkSize) as $_q) {

          // Get translation from DeepL Pro.
          $result = $this->deeplProRequestTranslation($job, $_q);

          // Collect translated texts with use of initial keys.
          foreach ($result['translations'] as $translated) {
            $translation[$keys_sequence[$i]]['#text'] = $translated['text'];
            $i++;
          }
        }

        // Save the translated data through the job.
        // NOTE that this line of code is reached only in case all translation
        // requests succeeded.
        $job_item->addTranslatedData($this->tmgmtData->unflatten($translation));

      }
      catch (TMGMTException $e) {
        $job->rejected('Translation has been rejected with following error: @error',
          ['@error' => $e->getMessage()], 'error');
      }
    }
  }

  /**
   * Local method to do request to DeepL Pro Usage service.
   *
   * @param \Drupal\tmgmt\Entity\Translator $translator
   *   The translator entity to get the settings from.
   *
   * @return array|object|integer
   *   Unserialized JSON response from DeepL Pro or error code.
   *
   * @throws TMGMTException
   *   - Unable to connect to the DeepL Pro Service
   *   - Error returned by the DeepL Pro Service.
   */
  public function getUsageData(Translator $translator) {
    // Set custom data for testing purposes, if available.
    $custom_usage_url = $translator->getSetting('url_usage');
    $custom_auth_key = $translator->getSetting('auth_key');

    $url = $custom_usage_url ? $custom_usage_url : $this->translatorUsageUrl;
    $auth_key = $custom_auth_key ? $custom_auth_key : $translator->getSetting('auth_key');

    // Prepare Guzzle Object.
    $request = new Request('GET', $url);

    // Build the query.
    $query_string = '&auth_key=' . $auth_key;

    // Send the request with the query.
    try {
      $response = $this->client->send($request, ['query' => $query_string]);
    }
    catch (BadResponseException $e) {
      return json_decode($e->getCode());
    }

    // Process the JSON result into array.
    return json_decode($response->getBody(), TRUE);
  }

}
